use std::ops::Deref;

use bpe_openai::Tokenizer;
use pyo3::prelude::*;

// enc = rtok.openai.Encoder(..)
// enc = rtok.openai.get_o200k_base()
// enc = rtok.openai.encoding_for_model('gpt-4o')
// enc.encode(str)

enum OwnedOrStatic {
    Owned(Tokenizer),
    Static(&'static Tokenizer),
}
impl Deref for OwnedOrStatic {
    type Target = Tokenizer;

    fn deref(&self) -> &Self::Target {
        match self {
            OwnedOrStatic::Owned(t) => t,
            OwnedOrStatic::Static(t) => t,
        }
    }
}

#[pyfunction]
fn get_o200k_base() -> Encoder {
    Encoder(OwnedOrStatic::Static(bpe_openai::o200k_base()))
}

#[pyfunction]
fn get_cl100k_base() -> Encoder {
    Encoder(OwnedOrStatic::Static(bpe_openai::cl100k_base()))
}

#[pyclass]
pub struct Encoder(OwnedOrStatic);

#[pymodule]
pub fn openai(m: &Bound<'_, PyModule>) -> PyResult<()> {
    m.add_class::<Encoder>()?;
    m.add_function(wrap_pyfunction!(get_o200k_base, m)?)?;
    m.add_function(wrap_pyfunction!(get_cl100k_base, m)?)?;
    Ok(())
}

#[pymethods]
impl Encoder {
    #[new]
    fn new() -> Self {
        todo!()
        // Encoder(OwnedOrStatic::Owned(Tokenizer::new_lookahead(bpe, patterns)))
    }

    fn count(&self, text: &str) -> usize {
        self.0.count(text)
    }

    pub fn count_till_limit(&self, text: &str, token_limit: usize) -> Option<usize> {
        self.0.count_till_limit(text, token_limit)
    }

    fn encode(&self, text: &str) -> Vec<u32> {
        self.0.encode(text)
    }

    fn decode(&self, tokens: Vec<u32>) -> Option<String> {
        self.0.decode(&tokens)
    }
}
